/*
    SPDX-FileCopyrightText: 2007 Vladimir Kuznetsov <ks.vladimir@gmail.com>
    SPDX-FileCopyrightText: 2014 Inge Wallin <inge@lysator.liu.se>

    SPDX-License-Identifier: GPL-2.0-or-later
*/

#include "objecterrors.h"


namespace StepCore
{

STEPCORE_META_OBJECT(ObjectErrors, QT_TRANSLATE_NOOP("ObjectClass", "ObjectErrors"), QT_TRANSLATE_NOOP("ObjectDescription", "ObjectErrors"),
		     MetaObject::ABSTRACT, STEPCORE_SUPER_CLASS(Object),)



} // namespace StepCore
