/****************************************************************************
** Resource object code
**
** Created by: The Resource Compiler for Qt version 6.11.2
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#ifdef _MSC_VER
// disable informational message "function ... selected for automatic inline expansion"
#pragma warning (disable: 4711)
#endif

static const unsigned char qt_resource_data[] = {
  // stepui.rc
  0x0,0x0,0x1,0xd8,
  0x28,
  0xb5,0x2f,0xfd,0x60,0x96,0x5,0x75,0xe,0x0,0xb6,0x93,0x3c,0x1e,0x40,0x73,0x1b,
  0x8c,0x48,0x49,0x4d,0x42,0x7e,0x81,0x8d,0x60,0x4,0xac,0x64,0x3f,0x9c,0xe1,0x10,
  0xc8,0xc6,0x36,0x7c,0xf6,0x57,0x52,0x3e,0xab,0x6b,0x4c,0x39,0x0,0x33,0x0,0x31,
  0x0,0x97,0x3e,0xd6,0x9a,0x7b,0x47,0xe0,0x69,0x7d,0x54,0x98,0x69,0x2c,0xf2,0x98,
  0x24,0x8b,0xc4,0x78,0x5a,0xec,0x31,0xb1,0x4c,0xd,0x89,0x23,0x0,0x1,0x22,0x45,
  0x3b,0xc,0x20,0x53,0xa5,0x25,0xac,0xc7,0xce,0xab,0xb7,0xac,0x15,0x8,0x86,0x96,
  0x78,0xce,0x32,0xae,0x86,0xf5,0xc5,0x51,0x40,0x1,0x2c,0x1f,0xe8,0xa7,0x3f,0xac,
  0x3d,0xa6,0x2e,0xf7,0xe,0xb5,0x59,0xd6,0x54,0x8b,0x7,0x8a,0x7a,0x61,0x8f,0x35,
  0x12,0x3d,0x67,0x9f,0xb5,0xa8,0x8c,0x76,0x9f,0x20,0xc6,0x0,0xd1,0x63,0xbd,0xdc,
  0xf1,0x9c,0x5f,0x5b,0x3e,0x8d,0xcf,0x1a,0xdc,0x97,0x2d,0xb1,0x1,0x94,0x68,0x7f,
  0x7b,0x1d,0xf5,0xec,0x1e,0x8,0x4e,0xa7,0x1b,0x77,0xad,0x95,0xf7,0x3,0x21,0x68,
  0x72,0x1b,0x5e,0xca,0xd9,0x85,0x4e,0xb5,0x72,0xf4,0x9f,0x4f,0x6d,0x33,0x66,0xbe,
  0x8d,0xb8,0x5d,0x38,0x23,0xf4,0x97,0xd4,0x47,0x7d,0x49,0x49,0xf3,0xa,0x80,0x88,
  0xa7,0xf5,0x40,0xee,0x4d,0xc7,0x3c,0x96,0xcb,0x1f,0x17,0x7a,0x84,0xae,0xa3,0x6f,
  0x32,0xd5,0xff,0xb6,0x42,0x6a,0xdb,0xb2,0xfc,0x27,0xbd,0x4f,0x21,0x6b,0xd2,0x7a,
  0xfc,0xbf,0x12,0xee,0xbf,0x8c,0xbb,0xf1,0x96,0xdd,0x53,0x62,0x6f,0x6,0x5e,0xa0,
  0x21,0x12,0x54,0x45,0x41,0x41,0x93,0x14,0x76,0xe6,0x20,0x44,0x40,0x10,0x65,0xd6,
  0xd,0xcd,0xa4,0xac,0x2d,0x73,0xcd,0xb9,0x2c,0x91,0x98,0x1b,0xbc,0x89,0x98,0x9a,
  0x6b,0x7e,0x58,0x44,0x53,0x6d,0x81,0x13,0x6b,0xa3,0xe2,0x10,0xb,0xa5,0x99,0x1a,
  0x12,0x73,0x1d,0x2a,0xe2,0x93,0xd2,0xdf,0x71,0x9b,0xbc,0x4c,0x94,0x4c,0x49,0x21,
  0xb8,0x56,0x8b,0x7b,0x5d,0x59,0xd0,0x24,0x19,0xe,0x83,0x6d,0x1b,0x5d,0x32,0x38,
  0xdf,0x18,0x67,0x72,0xf9,0x5,0x99,0x48,0x1c,0x66,0xb2,0x94,0xc1,0xbe,0x2e,0x45,
  0xe5,0x3a,0xb0,0x6e,0xe9,0xaf,0x33,0xc8,0xbe,0x19,0x6e,0xf1,0x6b,0x41,0x90,0xdb,
  0x4,0x3,0x7f,0x82,0x46,0x3b,0x9f,0xa1,0x74,0xc0,0xa7,0x94,0xdf,0x9d,0xef,0x1d,
  0xe3,0xc,0x5c,0x80,0x4d,0x64,0x90,0xdb,0x3b,0x30,0x4e,0xd1,0x80,0x59,0x24,0x48,
  0x3e,0x79,0xe2,0xcb,0x5e,0x96,0x53,0xc2,0xd9,0x76,0x63,0x3a,0xb9,0x5d,0xa7,0xf3,
  0x46,0x2,0x10,0x1,0x51,0x85,0xca,0x39,0x1e,0xd2,0x35,0x71,0xe2,0x66,0xa5,0x5f,
  0x2e,0x15,0xa4,0xdf,0xe7,0x3e,0xb3,0xc7,0x40,0x14,0xea,0xb3,0xb1,0x4,0x79,0x19,
  0xef,0xe9,0x56,0x25,0x78,0x60,0x58,0xe5,0xd0,0x5e,0x14,0x73,0x3a,0x97,0x39,0xde,
  0x68,0x81,0x4c,0x41,0x80,0xa9,0x2,
  
};

static const unsigned char qt_resource_name[] = {
  // kxmlgui5
  0x0,0x8,
  0xf,0x42,0xe5,0x85,
  0x0,0x6b,
  0x0,0x78,0x0,0x6d,0x0,0x6c,0x0,0x67,0x0,0x75,0x0,0x69,0x0,0x35,
    // step
  0x0,0x4,
  0x0,0x7,0xaa,0xc0,
  0x0,0x73,
  0x0,0x74,0x0,0x65,0x0,0x70,
    // stepui.rc
  0x0,0x9,
  0xc,0x7b,0x70,0xc3,
  0x0,0x73,
  0x0,0x74,0x0,0x65,0x0,0x70,0x0,0x75,0x0,0x69,0x0,0x2e,0x0,0x72,0x0,0x63,
  
};

static const unsigned char qt_resource_struct[] = {
  // :
  0x0,0x0,0x0,0x0,0x0,0x2,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x1,
0x0,0x0,0x1,0xa0,0x1a,0x8e,0xb0,0x50,
  // :/kxmlgui5
  0x0,0x0,0x0,0x0,0x0,0x2,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x2,
0x0,0x0,0x1,0xa0,0x1a,0x8e,0xb0,0x50,
  // :/kxmlgui5/step
  0x0,0x0,0x0,0x16,0x0,0x2,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x3,
0x0,0x0,0x1,0xa0,0x1a,0x8e,0xb0,0x50,
  // :/kxmlgui5/step/stepui.rc
  0x0,0x0,0x0,0x24,0x0,0x4,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x0,
0x0,0x0,0x1,0xa0,0x1a,0x8e,0xb0,0x50,

};

#ifdef QT_NAMESPACE
#  define QT_RCC_PREPEND_NAMESPACE(name) ::QT_NAMESPACE::name
#  define QT_RCC_MANGLE_NAMESPACE0(x) x
#  define QT_RCC_MANGLE_NAMESPACE1(a, b) a##_##b
#  define QT_RCC_MANGLE_NAMESPACE2(a, b) QT_RCC_MANGLE_NAMESPACE1(a,b)
#  define QT_RCC_MANGLE_NAMESPACE(name) QT_RCC_MANGLE_NAMESPACE2( \
        QT_RCC_MANGLE_NAMESPACE0(name), QT_RCC_MANGLE_NAMESPACE0(QT_NAMESPACE))
#else
#   define QT_RCC_PREPEND_NAMESPACE(name) name
#   define QT_RCC_MANGLE_NAMESPACE(name) name
#endif

#if defined(QT_INLINE_NAMESPACE)
inline namespace QT_NAMESPACE {
#elif defined(QT_NAMESPACE)
namespace QT_NAMESPACE {
#endif

bool qRegisterResourceData(int, const unsigned char *, const unsigned char *, const unsigned char *);
bool qUnregisterResourceData(int, const unsigned char *, const unsigned char *, const unsigned char *);

#if defined(__ELF__) || defined(__APPLE__)
static inline unsigned char qResourceFeatureZstd()
{
    extern const unsigned char qt_resourceFeatureZstd;
    return qt_resourceFeatureZstd;
}
#else
unsigned char qResourceFeatureZstd();
#endif

#ifdef QT_NAMESPACE
}
#endif

int QT_RCC_MANGLE_NAMESPACE(qInitResources_step)();
int QT_RCC_MANGLE_NAMESPACE(qInitResources_step)()
{
    int version = 3;
    QT_RCC_PREPEND_NAMESPACE(qRegisterResourceData)
        (version, qt_resource_struct, qt_resource_name, qt_resource_data);
    return 1;
}

int QT_RCC_MANGLE_NAMESPACE(qCleanupResources_step)();
int QT_RCC_MANGLE_NAMESPACE(qCleanupResources_step)()
{
    int version = 3;
    version += QT_RCC_PREPEND_NAMESPACE(qResourceFeatureZstd());
    QT_RCC_PREPEND_NAMESPACE(qUnregisterResourceData)
       (version, qt_resource_struct, qt_resource_name, qt_resource_data);
    return 1;
}

#ifdef __clang__
#   pragma clang diagnostic push
#   pragma clang diagnostic ignored "-Wexit-time-destructors"
#endif

namespace {
   struct initializer {
       initializer() { QT_RCC_MANGLE_NAMESPACE(qInitResources_step)(); }
       ~initializer() { QT_RCC_MANGLE_NAMESPACE(qCleanupResources_step)(); }
   } dummy;
}

#ifdef __clang__
#   pragma clang diagnostic pop
#endif
