/********************************************************************************
** Form generated from reading UI file 'configure_controller.ui'
**
** Created by: Qt User Interface Compiler version 6.11.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CONFIGURE_CONTROLLER_H
#define UI_CONFIGURE_CONTROLLER_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include <toolgraphics.h>
#include "kkeysequencewidget.h"

QT_BEGIN_NAMESPACE

class Ui_WidgetConfigureController
{
public:
    QVBoxLayout *vboxLayout;
    QGroupBox *groupBox;
    QGridLayout *gridLayout;
    DataSourceWidget *dataSource;
    QGroupBox *groupBox_2;
    QGridLayout *gridLayout1;
    QLabel *label_5;
    QLabel *label_6;
    QLineEdit *lineEditMin;
    QLineEdit *lineEditMax;
    QGroupBox *groupBox_3;
    QGridLayout *gridLayout2;
    QLabel *label;
    QLabel *label_2;
    KKeySequenceWidget *keyDecrease;
    KKeySequenceWidget *keyIncrease;
    QHBoxLayout *hboxLayout;
    QSpacerItem *spacerItem;
    QLabel *label_3;
    QLineEdit *lineEditIncrement;
    QSpacerItem *spacerItem1;

    void setupUi(QWidget *WidgetConfigureController)
    {
        if (WidgetConfigureController->objectName().isEmpty())
            WidgetConfigureController->setObjectName("WidgetConfigureController");
        WidgetConfigureController->resize(286, 318);
        vboxLayout = new QVBoxLayout(WidgetConfigureController);
        vboxLayout->setObjectName("vboxLayout");
        vboxLayout->setContentsMargins(0, 0, 0, 0);
        groupBox = new QGroupBox(WidgetConfigureController);
        groupBox->setObjectName("groupBox");
        gridLayout = new QGridLayout(groupBox);
        gridLayout->setObjectName("gridLayout");
        dataSource = new DataSourceWidget(groupBox);
        dataSource->setObjectName("dataSource");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(1);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(dataSource->sizePolicy().hasHeightForWidth());
        dataSource->setSizePolicy(sizePolicy);

        gridLayout->addWidget(dataSource, 0, 0, 1, 1);


        vboxLayout->addWidget(groupBox);

        groupBox_2 = new QGroupBox(WidgetConfigureController);
        groupBox_2->setObjectName("groupBox_2");
        gridLayout1 = new QGridLayout(groupBox_2);
        gridLayout1->setObjectName("gridLayout1");
        label_5 = new QLabel(groupBox_2);
        label_5->setObjectName("label_5");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(label_5->sizePolicy().hasHeightForWidth());
        label_5->setSizePolicy(sizePolicy1);
        label_5->setAlignment(Qt::AlignCenter);

        gridLayout1->addWidget(label_5, 0, 0, 1, 1);

        label_6 = new QLabel(groupBox_2);
        label_6->setObjectName("label_6");
        sizePolicy1.setHeightForWidth(label_6->sizePolicy().hasHeightForWidth());
        label_6->setSizePolicy(sizePolicy1);
        label_6->setAlignment(Qt::AlignCenter);

        gridLayout1->addWidget(label_6, 0, 1, 1, 1);

        lineEditMin = new QLineEdit(groupBox_2);
        lineEditMin->setObjectName("lineEditMin");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy2.setHorizontalStretch(1);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(lineEditMin->sizePolicy().hasHeightForWidth());
        lineEditMin->setSizePolicy(sizePolicy2);
        lineEditMin->setMaximumSize(QSize(100, 16777215));
        lineEditMin->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout1->addWidget(lineEditMin, 1, 0, 1, 1);

        lineEditMax = new QLineEdit(groupBox_2);
        lineEditMax->setObjectName("lineEditMax");
        sizePolicy2.setHeightForWidth(lineEditMax->sizePolicy().hasHeightForWidth());
        lineEditMax->setSizePolicy(sizePolicy2);
        lineEditMax->setMaximumSize(QSize(100, 16777215));
        lineEditMax->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout1->addWidget(lineEditMax, 1, 1, 1, 1);


        vboxLayout->addWidget(groupBox_2);

        groupBox_3 = new QGroupBox(WidgetConfigureController);
        groupBox_3->setObjectName("groupBox_3");
        gridLayout2 = new QGridLayout(groupBox_3);
        gridLayout2->setObjectName("gridLayout2");
        label = new QLabel(groupBox_3);
        label->setObjectName("label");
        label->setAlignment(Qt::AlignCenter);

        gridLayout2->addWidget(label, 0, 0, 1, 1);

        label_2 = new QLabel(groupBox_3);
        label_2->setObjectName("label_2");
        label_2->setAlignment(Qt::AlignCenter);

        gridLayout2->addWidget(label_2, 0, 1, 1, 1);

        keyDecrease = new KKeySequenceWidget(groupBox_3);
        keyDecrease->setObjectName("keyDecrease");

        gridLayout2->addWidget(keyDecrease, 1, 0, 1, 1);

        keyIncrease = new KKeySequenceWidget(groupBox_3);
        keyIncrease->setObjectName("keyIncrease");

        gridLayout2->addWidget(keyIncrease, 1, 1, 1, 1);

        hboxLayout = new QHBoxLayout();
        hboxLayout->setObjectName("hboxLayout");
        spacerItem = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(spacerItem);

        label_3 = new QLabel(groupBox_3);
        label_3->setObjectName("label_3");

        hboxLayout->addWidget(label_3);

        lineEditIncrement = new QLineEdit(groupBox_3);
        lineEditIncrement->setObjectName("lineEditIncrement");
        lineEditIncrement->setMaximumSize(QSize(100, 16777215));
        lineEditIncrement->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        hboxLayout->addWidget(lineEditIncrement);

        spacerItem1 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(spacerItem1);


        gridLayout2->addLayout(hboxLayout, 2, 0, 1, 2);


        vboxLayout->addWidget(groupBox_3);

        QWidget::setTabOrder(dataSource, lineEditMin);
        QWidget::setTabOrder(lineEditMin, lineEditMax);

        retranslateUi(WidgetConfigureController);

        QMetaObject::connectSlotsByName(WidgetConfigureController);
    } // setupUi

    void retranslateUi(QWidget *WidgetConfigureController)
    {
        groupBox->setTitle(tr2i18n("Data Source", nullptr));
        groupBox_2->setTitle(tr2i18n("Range", nullptr));
        label_5->setText(tr2i18n("min:", nullptr));
        label_6->setText(tr2i18n("max:", nullptr));
        lineEditMin->setText(tr2i18n("0", nullptr));
        lineEditMax->setText(tr2i18n("1", nullptr));
        groupBox_3->setTitle(tr2i18n("Shortcuts", nullptr));
        label->setText(tr2i18n("Decrease:", nullptr));
        label_2->setText(tr2i18n("Increase:", nullptr));
        label_3->setText(tr2i18n("Increment:", nullptr));
        lineEditIncrement->setText(tr2i18n("0.1", nullptr));
        (void)WidgetConfigureController;
    } // retranslateUi

};

namespace Ui {
    class WidgetConfigureController: public Ui_WidgetConfigureController {};
} // namespace Ui

QT_END_NAMESPACE

#endif // CONFIGURE_CONTROLLER_H

